/*
Author : Yongho, Ji ( http://blog.jidolstar.com, jidolstar@gmail.com )

License : MIT ( http://synczen.com/48 참고)

FAService is copyright 2008 Yongho, Ji.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
package com.jidolstar.fa
{
	import com.adobe.serialization.json.*;
	
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.external.ExternalInterface;
	import flash.utils.setTimeout;
	
	
	use namespace faservice_internal
	
	/**
	 * 자바스크립트 통신용 클래스
	 * 
	 * @author 지용호( http://blog.jidolstar.com, jidolstar@gmail.com )
	 * @since 2008.10.10
	 * @version 2008.10.13.
	 */ 
	public class FAService extends EventDispatcher
	{
		private static var jsInstenaceNameList:Array = [];
		
		private var isReady:Boolean = false;
		private var _faServiceJSInstanceName:String;
		
		private var flex2js:Flex2JS;
		private var js2flex:JS2Flex;
		
		private var _flex2jsCallerList:Array;
		private var _js2flexCallerList:Array;
		
		/**
		 * 생성자
		 * @param faServiceJSInstanceName Javascript FAService 클래스의 인스턴스명이다. 가령 자바스크립트 코드에서 var faService = new FAService(); 처럼 사용했다면 생성된 인스턴스 명인 "faService"이다.
		 * @param flex2jsCallerList Flex2JSCaller를 가지는 Array
		 * @param JS2FlexCallerList JS2FlexCaller를 가지는 Array		
		 */
		public function FAService( faServiceJSInstanceName:String = null, flex2jsCallerList:Array = null, js2flexCallerList:Array = null ):void
		{
			//중복되는 이름 방지.
			for each( var name:String in jsInstenaceNameList )
			{
				if( name == faServiceJSInstanceName )
				{
					throw new Error( "Duplicate FAService javascript Instance Name : " + faServiceJSInstanceName );
				}
			}
			jsInstenaceNameList.push( faServiceJSInstanceName );
			this.faServiceJSInstanceName = faServiceJSInstanceName;
			_flex2jsCallerList = flex2jsCallerList;
			_js2flexCallerList = js2flexCallerList; 

			flex2js = new Flex2JS( this );
			js2flex = new JS2Flex( this );
			if( flex2jsCallerList != null )
				flex2js.callerList = flex2jsCallerList.source;
			if( js2flexCallerList != null )
				js2flex.callerList = js2flexCallerList.source;
				
		}
		
		/**
		 * FAService를 실행한다.
		 */ 
		public function run():void
		{
			if( !isReady )
			{
				ExternalInterface.addCallback( "checkExternalInterface", onCheckExternalInterface );
				checkExternalInterface();
			}			
		}
		
		/**
		 * Javascript FAService 클래스의 인스턴스명이다. 
		 * 가령 자바스크립트 코드에서 var faService = new FAService(); 
		 * 처럼 사용했다면 생성된 인스턴스 명인 "faService"이다.
		 */ 
		[Bindable( event= "faServiceJSInstanceNameChanged" )]
		public function get faServiceJSInstanceName():String
		{
			return _faServiceJSInstanceName; 
		}
		
		public function set faServiceJSInstanceName( value:String ):void
		{
			if( _faServiceJSInstanceName == value ) return;
			_faServiceJSInstanceName = value;
			this.dispatchEvent( new Event("faServiceJSInstanceNameChanged" ) );
		}

		/**
		 * Flex2JSCaller를 가지는 Array
		 */ 		
		[Bindable( event= "flex2jsCallerListChanged" )]
		public function get flex2jsCallerList():Array
		{
			return _flex2jsCallerList;
		}
		
		public function set flex2jsCallerList( value:Array ):void
		{
			if( _flex2jsCallerList == value ) return;
			_flex2jsCallerList = value;
			if( value != null ) 
				flex2js.callerList = _flex2jsCallerList;
			this.dispatchEvent( new Event("flex2jsCallerListChanged" ) );
		}
		
		/**
		 * JS2FlexCaller를 가지는 Array
		 */ 		
		[Bindable( event= "js2flexCallerListChanged" )]
		public function get js2flexCallerList():Array
		{
			return _js2flexCallerList;
		}
		
		public function set js2flexCallerList( value:Array ):void
		{
			if( _js2flexCallerList == value ) return;
			_js2flexCallerList = value;
			if( value != null ) 
				js2flex.callerList = value;
			this.dispatchEvent( new Event("js2flexCallerListChanged" ));
		}
		
		/**
		 * @private
		 * ExternalInterface를 이용해 Flex-Javascript간에 통신이 가능한지 check한다. 
		 */ 
		private function checkExternalInterface():void
		{
			if( ExternalInterface.call( faServiceJSInstanceName + ".checkExternalInterface" ) )
			{
				ready();
			}
			else
			{
				setTimeout( checkExternalInterface, 200 );
			}
		}
		
		/**
		 * @private
		 * checkExternalInterface()함수에서 Javascript JSComm의 checkExternalInterface()를 통해서 호출되는 함수
		 * Flex->JS, JS->Flex라면 이 함수가 호출되어 ExternalInterface를 사용할 수 있는 환경이 만들어 진다.
		 */ 
		private function onCheckExternalInterface():Boolean
		{
			return true;	
		}		
		
		/**
		 * @private
		 * 통신이 가능하게 되면 호출된다. Javascript JSComm의 ready 함수를 호출한다.
		 * 통신이 불가능하기 전에 addEventListener()를 통해 리스너 함수를 등록했다면 JSComm의 ready 함수에서 그들을 처리해준다. 
		 */ 
		private function ready():void
		{
			isReady = true;
			ExternalInterface.addCallback( "call", onCallback );
			ExternalInterface.addCallback( "addEventListener", onAddEventListener );
			ExternalInterface.addCallback( "removeEventListener", onRemoveEventListener );
			this.dispatchEvent( new Event("ready") );
			ExternalInterface.call( faServiceJSInstanceName + ".ready" );
		}
		
		/**
		 * @private
		 * Javascript로 부터 JSComm의 call()함수로 접근했을때 호출된다. 호출명에 따라 처리할 수 있도록 한다.
		 */ 
		private function onCallback( name:String, objJSON:Object ):Object
		{
			trace( "onCallback() funcname="+name );
			if( !js2flex.isCallable( name ) ) 
			{
				ExternalInterface.call( faServiceJSInstanceName + ".wrongCall", "[Error] call() name=" + name + ",json="+ JSON.encode( objJSON )  );
				return null;
			}
			
			try 
			{
				var returnObj:Object = js2flex.call( name, objJSON );
				return returnObj;
			}
			catch( e:Error )
			{
				ExternalInterface.call( faServiceJSInstanceName + ".wrongCall", "[Error] call() wrong parameters. name=" + name + ",json="+JSON.encode( objJSON )  );
			}
			return null;
		}
		
		/**
		 * @private
		 * 자바스크립트 이벤트 리스너를 등록해준다.
		 */ 
		private function onAddEventListener( eventName:String, listenerName:String, priority:int = 0 ):void
		{
			if( !flex2js.addEventListener( eventName, listenerName, priority ) )
			{
				var msg:String = "[Error] addEventLister()"
								+ "eventName=" + eventName
								+ ", listenerName=" + listenerName
								+ ", priority=" + priority;
				
				ExternalInterface.call( faServiceJSInstanceName + ".wrongEventListener", msg );
				trace( "addedEventListener() fail");
			}
			else
			{
				trace( "addedEventListener() eventName=" + eventName + ", listenerName=" + listenerName + ", priority=" + priority );
			}
			
		}
		
		/**
		 * @private
		 * 자바스크립트 이벤트 리스너를 삭제한다.
		 */ 
		private function onRemoveEventListener( eventName:String, listenerName:String ):void
		{
			flex2js.removeEventListener( eventName, listenerName );
		}		

	}
}
