package com.jidolstar.fa
{
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.external.ExternalInterface;
		
	use namespace faservice_internal;
	
	/**
	 * Javascript에서 addEventListner()를 등록한 리스너 함수를 
	 * Flex에서 이벤트가 발생할 때  등록한 리스너 함수를 호출해주는 클래스
	 * 
	 * <p>
	 * 이 함수는 외부에서 직접 사용하지 않지 않으며 JSComm.setup() 호출시 
	 * 자동으로 com.jidolstar.fa.JSCaller 형태의 Array값을 받아서 처리한다.
	 * 
	 * <p>
	 * 이벤트가 발생할때 com.jidolstar.fa.Flex2JSCaller에 등록된 
	 * 이벤트 발생자, 이벤트 명, 이벤트 처리 함수를 이용하여 Javascript 함수에 값을 넘겨준다.
	 * 이 클래스를 사용하기 위해 사용자는 com.jidolstar.fa.Flex2JSCaller로 만들어진 Array를 미리 만들어야 한다.
	 * 
	 * @author 지용호( http://blog.jidolstar.com, jidolstar@gmail.com )
	 * @since 2008.10.10 
	 */ 
	internal class Flex2JS
	{
		private var jsEventListenerManager:JSEventListenerManager;
		private var faService:FAService;
		
		[ArrayElementType("com.jidolstar.fa.Flex2JSCaller")]		
		private var _callerList:Array;
		
		/**
		 * JSCaller 인스턴스를 가지는 Array를 등록한다.
		 * @param callerList JSCaller 인스턴스를 가지는 Array
		 */		
		public function Flex2JS( faService:FAService ):void
		{
			this._callerList = callerList;
			this.faService = faService;
			jsEventListenerManager = new JSEventListenerManager();			
		}
		
		public function get callerList():Array
		{
			return _callerList;
		}
		
		public function set callerList( value:Array ):void
		{
			_callerList = value;
			jsEventListenerManager.setCallerList( value );
		}
		
		/**
		 * Javascript 이벤트 등록
		 * @param eventName 이벤트 명
		 * @param listenerName Javascript의 이벤트 리스너
		 * @param priority  우선순위, 숫자가 클수록 우선순위가 크다. 
		 * 					같은 이벤트명을 가지는 이벤트 리스너가 2개 이상인 경우  
		 * 					등록된 순서에 상관없이 우선순위가 높은 것이 이벤트 발생시 먼저 호출된다.
		 * 					우선순위가 같은 경우에는 등록된 순서대로 호출된다. 
		 * @return 등록에 성공 true, 이미 등록되어 있어도 true, 실패하면 false(받을 수 있는 이벤트가 아닌 경우)
		 */ 
		public function addEventListener( eventName:String, listenerName:String, priority:int = 0 ):Boolean
		{
			if( !jsEventListenerManager.addEventListener( eventName, listenerName, priority ) ) return false;
			
			//이벤트가 처음으로 생겼다면 Flex쪽 이벤트 리스너를 등록해준다. 
			if( jsEventListenerManager.getListenerCount( eventName ) == 1 )
			{
				var target:IEventDispatcher = getEventDispatcher( eventName );
				if( target == null ) return false;				
				target.addEventListener( eventName, jsCall );
			}
			return true;
		}
		
		/**
		 * Javascript 이벤트 삭제
		 * @param eventName 이벤트 명
		 * @param listenerName Javascript의 이벤트 리스너
		 * @return 삭제에 성공하면 true, 실패하면 false
		 */ 
		public function removeEventListener( eventName:String, listenerName:String ):Boolean
		{
			jsEventListenerManager.removeEventListener( eventName, listenerName );
			
			//등록된 이벤트가 더이상 없다면 Flex쪽 이벤트 리스너를 삭제한다.
			if( jsEventListenerManager.getListenerCount( eventName ) == 0 )
			{
				var target:IEventDispatcher = getEventDispatcher( eventName );
				if( target == null ) return false;
				target.removeEventListener( eventName, jsCall );
			}
			return true;
		}
		
		/**
		 * @private
		 */ 
		private function getEventDispatcher( eventName:String ):IEventDispatcher
		{
			for each( var jsCaller:Flex2JSCaller in _callerList )
			{
				if( jsCaller.eventName == eventName )
				{
					return jsCaller.eventDispatcher;
				}
			}
			return null;			
		}
		
		/**
		 * @private
		 */ 		
		private function getParamMaker( eventName:String ):Function
		{
			for each( var jsCaller:Flex2JSCaller in _callerList )
			{
				if( jsCaller.eventName == eventName )
				{
					return jsCaller.paramMaker;
				}
			}
			return null;			
		}
		
		/**
		 * @private
		 * 이벤트가 발생할때 처리하는 함수이다. 해당 Javascript 이벤트 리스너를 찾아 호출해준다.
		 */ 		
		private function jsCall( event:Event ):void
		{
			var jsListenerNames:Array = jsEventListenerManager.getListenerNames( event.type );
			var paramMaker:Function = getParamMaker( event.type );
			try
			{
				for each( var jslistenerName:String in jsListenerNames )
				{
					ExternalInterface.call( faService.faServiceJSInstanceName + ".calladdedEventListener", jslistenerName, paramMaker( event ) );
				}
			}
			catch( e:Error )
			{
				var msg:String = "Call Failed! added javascript Listener Names : [" + jsListenerNames.toString() + "] Flex Event Type = " + event.type;
				trace(msg);
				throw new Error( msg ); 
			}					
		}
	}
}

import com.jidolstar.fa.Flex2JSCaller;


/**
 * 이벤트를 관리하고 자바스크립트 이벤트 리스너 함수를 등록하는 역할을 한다.
 * 
 * @author Jidolstar 
 * @since 2008.10.09
 */ 
class JSEventListenerManager
{
	private var eventNameList:Object;
	
	public function JSEventListenerManager()
	{
		eventNameList = {};
	}
	
	public function setCallerList( value:Array ):void
	{
		eventNameList = {};
		for each( var jsCaller:Flex2JSCaller in value )
		{
			eventNameList[ jsCaller.eventName ] = new JSEventListenerInfo( jsCaller.eventName );
		}		
	}
	
	public function addEventListener( eventName:String, listenerName:String, priority:int = 0 ):Boolean
	{
		if( eventNameList[eventName] == null ) return false;
		if( listenerName == null ) return false;
		var eventListenerInfo:JSEventListenerInfo = eventNameList[eventName] as JSEventListenerInfo;
		if( eventListenerInfo == null ) return false;
		return eventListenerInfo.addEventListener( listenerName, priority );
	}
	
	public function removeEventListener( eventName:String, listenerName:String ):Boolean
	{
		if( eventNameList[eventName] == null ) return false;
		var eventListenerInfo:JSEventListenerInfo = eventNameList[eventName] as JSEventListenerInfo;
		if( eventListenerInfo == null) return false;
		return eventListenerInfo.removeEventListener( listenerName );
	}
	
	public function getListenerNames( eventName:String ):Array
	{
		if( eventNameList[eventName] == null ) return null;
		var eventListenerInfo:JSEventListenerInfo = eventNameList[eventName] as JSEventListenerInfo;
		if( eventListenerInfo == null) return null;
		return eventListenerInfo.getListenerNames();
	}
	
	public function getListenerCount( eventName:String ):int
	{
		if( eventNameList[eventName] == null ) return 0;
		var eventListenerInfo:JSEventListenerInfo = eventNameList[eventName] as JSEventListenerInfo;
		if( eventListenerInfo == null) return 0;
		return eventListenerInfo.getListenerCount();		
		
	}
}

/**
 * 하나의 이벤트로 호출할 자바스크립트 이벤트 리스너 함수들을 관리한다.
 *  
 * @author Jidolstar 
 * @since 2008.10.09
 */ 
class JSEventListenerInfo
{
	private var eventName:String;
	private var listeners:Array  = [];
	private var listenerNames:Array = [];
	public function JSEventListenerInfo( eventName:String )
	{
		this.eventName = eventName;
	}
	
	public function addEventListener( listenerName:String, priority:int = 0 ):Boolean
	{
		if( listenerName == null || listenerName == "" ) return false;
		
		//중복되는 이벤트 리스너가 있는지 확인
		for each( var listener:JSEventListener in listeners )
		{
			if( listener.listenerName == listenerName )
			{
				return true;
			}
		}
		
		//이벤트 리스너 등록
		listeners.push( new JSEventListener( listenerName, priority ) );
		
		//우선순위별로 sorting
		listeners.sortOn("priority", Array.NUMERIC | Array.DESCENDING );
		
		//이벤트 리스너 이름 배열 만듬
		makeListenerNames();
		return true;
	}
	
	public function removeEventListener( listenerName:String ):Boolean
	{
		var i:int = 0;
		var isFinded:Boolean = false;
		for each( var listener:JSEventListener in listeners )
		{
			if( listener.listenerName == listenerName )
			{
				listeners.splice( i, 1 );
				isFinded = true;
				break; 
			}
			i++;
		}
		makeListenerNames();
		return isFinded;
	}
	
	public function getListenerNames():Array
	{
		return listenerNames;
	}
	
	public function getListenerCount():int
	{
		return listeners.length;
	}
	
	private function makeListenerNames():void
	{
		listenerNames = [];
		for each( var listener:JSEventListener in listeners )
		{
			listenerNames.push( listener.listenerName );
		}
	}
}

/**
 * 자바스크립트 이벤트 리스너 정보 
 * 
 * @author Jidolstar 
 * @since 2008.10.09
 */ 
class JSEventListener
{
	public var listenerName:String;
	public var priority:int;
	public function JSEventListener( listenerName:String, priority:int )
	{
		this.listenerName = listenerName;
		this.priority = priority;
	}
}