package com.jidolstar.fa
{
	use namespace faservice_internal;
	
	/**
	 * Javascript에서 call() 함수로 접근할때 처리하는 클래스 
	 * 
	 * <p>
  	 * 이 함수는 외부에서 직접 사용하지 않지 않으며 JSComm.setup() 호출시 
	 * 자동으로 com.jidolstar.fa.FlexCaller 형태의 Array값을 받아서 처리한다.
	 * 
	 * <p>
	 * Javascript에서 call(호출명,JSON값) 메소드를 통해 호출할 때  
	 * com.jidolstar.fa.JS2FlexCaller에 등록된 호출명을 이용해 처리함수가 있는지 확인하고 
	 * 함수가 있다면 처리후 Javascript 함수에 값을 넘겨준다.
	 * 
	 * <p>
	 * <p>
	 * 이 클래스를 사용하기 위해 사용자는 com.jidolstar.fa.JS2FlexCaller로 만들어진 Array를 미리 만들어야 한다.
	 * 
	 * @author 지용호( http://blog.jidolstar.com, jidolstar@gmail.com )
	 * @since 2008.10.10
	 */ 	
	internal class JS2Flex
	{
		private var faService:FAService;
		
		[ArrayElementType("com.jidolstar.fa.JS2FlexCaller")]		
		private var _callerList:Array;
			
		/**
		 * FlexCaller 인스턴스를 가지는 Array를 등록한다.
		 * @param callerList FlexCaller 인스턴스를 가지는 Array
		 */ 	
		public function JS2Flex( faService:FAService ):void
		{
			this.faService = faService;
		}
		
		public function get callerList():Array
		{
			return _callerList;
		}
		
		public function set callerList( value:Array ):void
		{
			_callerList = value;
		}
		
		/**
		 * 호출할 수 있는 함수가 있는지 확인 
		 * @name 호출명
		 * @return 함수가 있다면 true
		 */ 
		public function isCallable( name:String ):Boolean
		{
			var flexCaller:JS2FlexCaller;
			for each( flexCaller in callerList )
			{
				if( name == flexCaller.name )
				{
					return true;
				}
			}			
			return false;
		}
		
		/**
		 * 등록된 함수를 찾아 호출한다.
		 * @name 호출명
		 * @params Object 형태의 인수 (사용자가 정하기 나름)
		 * @return Object 형태의 반환값 (사용자가 정하기 나름)
		 */ 
		public function call( name:String, params:Object ):Object
		{
			var flexCaller:JS2FlexCaller;
			for each( flexCaller in callerList )
			{
				if( name == flexCaller.name )
				{
					return flexCaller.caller( params );
				}
			}
			return null;
		}

	}
}