package com.jidolstar.fileupload
{
	import flash.net.FileReference;
	import flash.net.FileReferenceList;

	/**
	 * 여러개 파일의 정보들을 관리하는 클래스  
	 */ 
	public class FileDetailList
	{
		private var _fileList:Array;
		private var _fileTotalSize:Number = 0;
		private var _maxFileCount:int = 0;
		private var _maxOneFileSize:Number = 0;
		private var _currentFileDetailIndex:int = 0;
		private var _currentOneFileSize:Number = 0;
		
		public function FileDetailList()
		{
		}
		
		/**
		 * 파일 정보를 등록한다.
		 * @param fileReferenceList
		 */ 
		internal function registerFileList( fileReferenceList:FileReferenceList ):void
		{
			_fileList = [];
			if( fileReferenceList == null ) return;
			
			_fileTotalSize = 0;
			for each( var file:FileReference in fileReferenceList.fileList )
			{
				var fileDetail:FileDetail = new FileDetail;
				fileDetail.filename = file.name;
				fileDetail.bytesTotal = file.size;
				_fileTotalSize += file.size;
				_fileList.push( fileDetail );
			}
		} 
		
		/**
		 * 파일의 정보를 업데이트 한다. 
		 * @param file 
		 * @param bytesLoaded
		 * @param isFailed
		 * @param message 
		 * @param uploadCompleteData
		 */ 
		internal function updateFileDetail( file:FileReference, bytesLoaded:Number, isFailed:Boolean, message:String, uploadCompleteData:Object ):void
		{
			var i:int;
			for( i=0; i < _fileList.length; i++ ) 
			{
				var fileDetail:FileDetail = _fileList[i];
				if( fileDetail.filename == file.name ) 
				{
					if( !isNaN( bytesLoaded ) )
					{
						fileDetail.bytesLoaded = bytesLoaded;
					}
					
					fileDetail.isFailed = isFailed;
					
					if( message != null )
					{
						fileDetail.message = message;
					}
					
					if( uploadCompleteData )
					{
						fileDetail.uploadCompleteData = uploadCompleteData;
					}
					_currentFileDetailIndex = i;
					break;
				}
			}
		}
		
		/**
		 * 등록된 파일 정보 리스트를 반환한다. 
		 */ 
		public function get fileList():Array
		{
			return _fileList;
		}
		
		/**
		 * 업로드 실패한 파일 수 
		 */ 
		public function get uploadFailCount():int
		{
			var count:int = 0;
			for each( var file:FileDetail in _fileList )
			{
				if( file.isFailed )
					count++;
			}
			return count;
		}
		
		/**
		 * 업로드가 완료된 파일 수 
		 */ 
		public function get uploadEndCount():int
		{
			var count:int = 0;
			for each( var file:FileDetail in _fileList )
			{
				if( !file.isFailed )
					count++;
			}
			return count;				
		}
		
		/**
		 * 현재 업로드 진행중인 파일 
		 */ 
		public function get currentFileDetail():FileDetail
		{
			return _fileList[_currentFileDetailIndex];
		}
		
		/**
		 * 등록된 파일 수 
		 */ 
		public function get fileTotalCount():int
		{
			if( _fileList == null )
				return 0;
			else
				return _fileList.length;
		}
		
		/**
		 * 현재 업로드 중인 파일 사이즈 
		 */ 
		public function get fileTotalSize():int
		{
			return _fileTotalSize;
				
		}
		
		/**
		 * 최대 파일 갯수 
		 */ 
		public function get maxFileCount():int
		{
			return _maxFileCount;
		}

		/**
		 * @private
		 */ 		
		public function set maxFileCount( value:int ):void
		{
			_maxFileCount = value;
		}
		
		/**
		 * 한개 파일의 제한 크기 
		 */ 
		public function get maxOneFileSize():Number 
		{
			return _maxOneFileSize;
		}
		
		/**
		 * @private
		 */ 
		public function set maxOneFileSize( value:Number ):void
		{
			_maxOneFileSize = value;
		}
		
		/**
		 * 파일을 제한된 갯수검사
		 * @return 파일을 제한된 갯수보다 많이 선택했을때 false
		 */ 
		internal function isFileCountError():Boolean
		{
			if( _fileList.currentFileCount > _fileList.maxFileCount )
			{
				return true;
			}		
			return false;
		}
		
		/**
		 * 해당 파일의 크기 검사 
		 * @param file
		 * @return 해당 파일의 크기가 정해진 크기보다 큰 경우 false
		 */ 
		internal function isOneFileSizeError( file:FileReference ):Boolean
		{
			if( file.size > _maxOneFileSize )
			{
				var i:int;
				for( i=0; i < _fileList.length; i++ ) 
				{
					if ( file == _fileList[i] )
					{
						_currentFileDetailIndex = i;
						break;
					}
				}				
				return true;
			}
			return false;
		}

	}
}