package com.jidolstar.fileupload
{
	import com.adobe.serialization.json.JSON;
	
	import flash.events.DataEvent;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.FileReference;
	import flash.net.FileReferenceList;
	import flash.net.URLRequest;
	import flash.net.URLVariables;
	
	/**
	 * <pre>
	 * 파일 업로드 코어. 
	 * FileReferenceList와 FileReference를 이용해서 여러개의 파일을 서버에 올리고 이벤트를 송출해주는 역할을 담당한다.
	 * 이것은 자바스크립트 통신을 하지 않고 Flex에서 단독으로 사용하려면 이 클래스를 이용하면 되겠다. 
	 * </pre>
	 * @author Yongho, Ji (jidolstar@gmail.com, http://blog.jidolstar.com)
	 * @since 2008.10.24.
	 * @version 2008.11.14.
	 */ 
	public class FileUploadCore extends EventDispatcher
	{
		private var _uploadURL:String;	//업로드 URL
		
		private var _fileFilterList:Array;	//File Filter 들 
		private var _banFileExtensionList:Array; //제한할 확장자 들 
		
		private var _uploadDataFieldName:String = "FileData"; //Upload시 파일의 DataFile명
		private var _requestHeaders:Object;	//HTTP 리퀘스트 헤더
		private var _variables:URLVariables; //HTTP로 서버측에 보낼 데이터 
		private var _method:String = "post"; //HTTP 폼 송신 Methods. GET 또는 POST
		private var _contentType:String; //임의의 POST 데이터의 MIME 컨텐츠 타입입니다. 
		
		private var fileRefList:FileReferenceList; 
		private var pendingFiles:Array; //upload중인 FileReference를 담는 Array
		private var urlRequest:URLRequest; //업로드 URL에 대한 URLRequest
		
		private var isUploading:Boolean = false; //업로드 중일때 true
		private var uploadEndCount:int = 0; //최종 업로드를 완료한 파일 수 
		private var uploadFailCount:int = 0; //업로드 실패한 파일 수
		private var uploadTotalCount:int = 0; //업로드 시도 파일 수 
		private var uploadTotalSize:Number = 0; //업로드하는 파일의 전체 용량 

		private var fileDetailList:FileDetailList;

		/**
		 * 생성자, FileReferenceList 생성 및 이벤트 핸들러 등록 
		 */ 
		public function FileUploadCore()
		{
			fileRefList = new FileReferenceList();
			fileDetailList = new FileDetailList();
		}
		
		/**
		 * 다중 파일 브라우징 실시. 이때 FileFilter를 적용한다. 
		 */ 
		public function browse():void
		{
			fileRefList.browse( _fileFilterList );
			fileRefList.addEventListener(Event.SELECT, onSelect );
			fileRefList.addEventListener(Event.CANCEL, onCancel );
		}
		
		/**
		 * 업로드 중지. 업로드 시작 전, 완료 전에는 동작하지 않음 
		 * FileUploadEvent.UPLOAD_STOP_EVENT를 발생하며 이 이벤트는  {"totalCount":업로드 파일 총수, "totalSize":업로드 파일 전체 용량} 데이타를 담는다. 
		 */ 
		public function stop():void
		{
			if( !isUploading ) return;
			
			isUploading = false;
			
			for each( var file:FileReference in pendingFiles )
			{
		        file.removeEventListener(Event.OPEN, onOpen);
		        file.removeEventListener(IOErrorEvent.IO_ERROR, onIOError);
		        file.removeEventListener(ProgressEvent.PROGRESS, onProgress);
		        file.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, onSecurityError);
		        file.removeEventListener(DataEvent.UPLOAD_COMPLETE_DATA, onUploadCompleteData );
			}
	        this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_STOP_ALL, fileDetailList ) );
		}
		
		/**
		 * @private
		 * 파일 선택했을때 처리하는 이벤트 핸들러 
		 */ 
		private function onSelect( event:Event ):void
		{
			var file:FileReference;
			
			//제한된 파일확장자 검사 
			if( _banFileExtensionList )
			{
				for each( file in fileRefList.fileList )
				{
					var extension:String = file.name.substr( file.name.lastIndexOf(".")+1, file.name.length );
					if( !extension ) continue;
					extension = extension.toLowerCase();
					for each( var banExtension:String in _banFileExtensionList )
					{
						if( banExtension == extension )
						{
							var data:Object = { "filename": file.name, 
												"banFileExtension": banExtension, 
												"banFileExtensions": _banFileExtensionList.join( ";" ) 
											  };
							this.dispatchEvent( new FileUploadEvent( FileUploadEvent.BAN_FILE_EXTENSION, data ) );
							return;
						}	
					}
				}
				
			}
			
			//파일 리스트 등록 
			fileDetailList.registerFileList( fileRefList );
			
			//파일 제한 갯수 검사 
			if( fileDetailList.isFileCountError() )
			{
				this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_FILE_COUNT_ERROR, fileDetailList ) );
				return;
			}
			
			//파일 업로드 준비 
			urlRequest = new URLRequest( uploadURL );
			if( method )
			{
				if( method.toLowerCase() == "get" || method.toLowerCase() == "post" )
				{
					urlRequest.method = method.toLowerCase();
				}
			}
			if( variables )
			{
				urlRequest.data = variables;
			}
			if( contentType )
			{
				urlRequest.contentType = contentType;
			}
			if( requestHeaders )
			{
				urlRequest.requestHeaders;
			}			
			
			//한개의 파일 사이즈 검사하면서 업로드 시행 
			pendingFiles = [];
			for each( file in fileRefList.fileList )
			{
				if( fileDetailList.isOneFileSizeError( file ) ) 
				{
					this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_FILE_SIZE_ERROR, fileDetailList ) );
					return;
				}
	            addPendingFile(file);
	        }							
			
			isUploading = true;
			
	        this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_START_ALL, fileDetailList ) );
	        uploadFailCount = 0;
	        uploadEndCount = 0;
		}

		/**
		 * @private
		 * 선택한 파일들을 하나씩 업로드시작
		 */ 
	    private function addPendingFile(file:FileReference):void 
	   	{
	        pendingFiles.push(file);
	        
	        file.addEventListener(Event.OPEN, onOpen);
	        file.addEventListener(IOErrorEvent.IO_ERROR, onIOError);
	        file.addEventListener(ProgressEvent.PROGRESS, onProgress);
	        file.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onSecurityError);
	        file.addEventListener(DataEvent.UPLOAD_COMPLETE_DATA, onUploadCompleteData );
	        file.upload(urlRequest, uploadDataFieldName );
	    }
		
		/**
		 * @private
		 * 파일 브라우징 취소  처리 핸들러 
		 */ 		
		private function onCancel( event:Event ):void
		{
			this.dispatchEvent( new FileUploadEvent( FileUploadEvent.CANCEL, null ) );
		}
		

		/**
		 * @private
		 * 파일 전송 시작  처리 핸들러 
		 */ 		
		private function onOpen( event:Event ):void
		{
			var file:FileReference = FileReference(event.target);
			fileDetailList.updateFileDetail( file, 0, false, null, null );
			this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_START, fileDetailList ) );
		}

		/**
		 * @private
		 * 파일 전송 중  처리 핸들러 
		 */ 		
		private function onProgress( event:ProgressEvent ):void
		{
			var file:FileReference = FileReference(event.target);
			fileDetailList.updateFileDetail( file, event.bytesLoaded, false, null, null );
			this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_STEP, fileDetailList ) );
		}
		
		/**
		 * @private
		 * 파일 전송 IO 에러 처리 핸들러 
		 */ 		
		private function onIOError( event:IOErrorEvent ):void
		{
			var file:FileReference = FileReference(event.target);
			uploadFailCount++;
			fileDetailList.updateFileDetail( file, NaN, true, event.text, null );
			this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_FAIL, fileDetailList ) );
			confirmUploadComplete();
		}

		/**
		 * @private
		 * 파일 전송 보안 에러 처리 핸들러 
		 */ 		
		private function onSecurityError( event:SecurityErrorEvent ):void
		{
			var file:FileReference = FileReference(event.target);
			uploadFailCount++;
			fileDetailList.updateFileDetail( file, NaN, true, event.text, null );
			this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_FAIL, fileDetailList ) );
			confirmUploadComplete();
		}

		/**
		 * @private
		 * 1개 파일 전송 완료하고 서버로부터 데이타가 넘어온 경우 처리 이벤트 핸들러  
		 */ 		
		private function onUploadCompleteData( event:DataEvent ):void
		{
			var file:FileReference = FileReference(event.target);
			uploadEndCount++;
			
			//JSON String형태의 경우 Object로 반환시켜준다. 만약 Object로 변환할 수 없을 경우 데이타를 그대로 반환한다.
			try
			{
				var data:Object = JSON.decode( event.data );
				fileDetailList.updateFileDetail( file, NaN, false, null, data );
				this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_END, fileDetailList ) );
				confirmUploadComplete();
			}
			catch( e:Error )
			{
				fileDetailList.updateFileDetail( file, NaN, false, null, event.data );
				this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_END, fileDetailList ) );
				confirmUploadComplete();
			}
		}

		/**
		 * @private
		 * 모든 파일 업로드가 완료되었을 경우 호출되는 함수 
		 */ 		
		private function confirmUploadComplete():void
		{
			if( uploadFailCount + uploadEndCount >= uploadTotalCount )
			{
				this.dispatchEvent( new FileUploadEvent( FileUploadEvent.UPLOAD_END_ALL, fileDetailList ) );
				isUploading = false;
			}
		}

		
		/**
		 * 업로드 URL 지정 
		 */ 		
		public function get uploadURL():String
		{
			return _uploadURL;
		}

		public function set uploadURL( value:String ):void
		{
			if( value == _uploadURL ) return;
			_uploadURL = value;
		}
		
		/**
		 * 1개 파일당 업로드 최대 용량 byte단위로 지정 
		 */ 		
		public function get maxOneFileSize():int
		{
			return fileDetailList.maxOneFileSize;
		}
		
		public function set maxOneFileSize( value:int ):void
		{
			if( value == fileDetailList.maxOneFileSize ) return;
			fileDetailList.maxOneFileSize = value;
		}
		
		/**
		 * 업로드 할 수 있는 파일 최대 갯수  
		 */ 		
		public function get maxFileCount():int
		{
			return fileDetailList.maxFileCount;
		}
		
		public function set maxFileCount( value:int ):void
		{
			if( value == fileDetailList.maxFileCount ) return;
			fileDetailList.maxFileCount = value;
		}

		/**
		 * 업로드시 데이타 필드 명
		 */ 		
		public function get uploadDataFieldName():String
		{
			return _uploadDataFieldName;
		}
		
		public function set uploadDataFieldName( value:String ):void
		{
			if( _uploadDataFieldName == value ) return;
			_uploadDataFieldName = value;
		}
		
		/**
		 * 파일 필터 리스트
		 */ 		
		public function get fileFilterList():Array
		{
			return _fileFilterList;
		}
		
		public function set fileFilterList( value:Array ):void
		{
			if( value == _fileFilterList ) return;
			_fileFilterList = value;
		}
		
		public function set banFileExtensionList( value:Array ):void
		{
			if( value == _banFileExtensionList ) return;
			
			var banExtension:String
			var list:Array = [];
			for each( banExtension in value )
			{
				if( !banExtension ) continue;
				if( banExtension.length == 0 ) continue;
				list.push( banExtension.toLowerCase() );
			}
			_banFileExtensionList = list;
		}
		
		public function get method():String
		{
			return _method;
		}
		
		public function set method( value:String ):void
		{
			if( value == _method ) return;
			_method = value;	
		}
		
		public function get variables():URLVariables
		{
			return _variables;
		}
		
		public function set variables( value:URLVariables ):void
		{
			if( value == _variables ) return;
			_variables = value;
		}
		
		public function get requestHeaders():Object
		{
			return _requestHeaders;
		}
		
		public function set requestHeaders( value:Object ):void
		{
			if( value == _requestHeaders ) return;
			_requestHeaders = value;
		}
		
		public function get contentType():String
		{
			return _contentType;
		}
		
		public function set contentType( value:String ):void
		{
			if( value == _contentType ) return;
			_contentType = value;
		}
				
	}
}