package com.jidolstar.fileupload
{
	import com.adobe.serialization.json.JSON;
	import com.jidolstar.fa.FAService;
	
	import flash.display.LoaderInfo;
	import flash.events.Event;
	import flash.net.FileFilter;
	import flash.net.URLRequestHeader;
	import flash.net.URLVariables;
	
	/**
	 * <pre>
	 * FileUploader는 자바스크립트와 함께 파일업로드를 제어하거나 진행상황을 자바스크립트에 전달하기 위한 목적으로 쓰인다.
	 * 
	 * </pre>
	 */ 
	public class FileUploader
	{
		private var uploadCore:FileUploadCore;
		private var faservice:FAService;
		private var f2jFunctions:Flex2JSFunctions;
		private var j2fFunctions:JS2FlexFunctions;
		
		
		/**
		 * 파일 업로더 생성자 
		 * @param loaderInfo LoaderInfo
		 */ 
		public function FileUploader( loaderInfo:LoaderInfo  )
		{
			////////////////////////////
			//다중 업로더 코어  						
			////////////////////////////
			uploadCore = new FileUploadCore();
			
			////////////////////////////
			//Flash-Ajax 통신 서비스
			////////////////////////////
			faservice = new FAService();
			f2jFunctions = new Flex2JSFunctions();
			j2fFunctions = new JS2FlexFunctions()
			f2jFunctions.target = uploadCore;
			j2fFunctions.target = uploadCore;
			faservice.flex2jsCallerList = f2jFunctions.callerList;
			faservice.js2flexCallerList = j2fFunctions.callerList; 

			
			////////////////////////////
			// Flash Vars을 이용한 설정
			////////////////////////////
			var params:Object = loaderInfo.parameters;

			//파일 필터 확장자 설명 Array
			var fileFilterDescriptions:Array;
			if( params.fileFilterDescriptions != null )
			{
				fileFilterDescriptions = String(params.fileFilterDescriptions).split("|");
			}
			
			//파일 필터 확장자 Array
			var fileFilterExtensions:Array;
			if( params.fileFilterExtensions != null )
			{
				fileFilterExtensions = String(params.fileFilterExtensions).split("|");
			}
			
			//파일 필터 제작 
			if( fileFilterDescriptions != null && fileFilterExtensions != null )
			{
				if( fileFilterDescriptions.length == fileFilterExtensions.length && fileFilterDescriptions.length > 0 )
				{
					var fileFilterList:Array = [];	
					var fileFilter:FileFilter;
					var length:int = fileFilterDescriptions.length;
					for( var i:int=0; i < length; i++ )
					{
						fileFilter = new FileFilter( fileFilterDescriptions[i], fileFilterExtensions[i] );
						fileFilterList.push( fileFilter );
					}
					uploadCore.fileFilterList = fileFilterList;
				}
			}
			
			//제한된 파일 확장자 
			if( params.banfileExtensions )
			{
				uploadCore.banFileExtensionList = String(params.banfileExtensions).split(";");
			}			
			
			//파일 1개 최대 사이즈(kb)
			uploadCore.maxOneFileSize = parseInt( params.maxOneFileSize );
			
			//중복 선택 최대갯수
			uploadCore.maxFileCount = parseInt( params.maxFileCount );
			
			//업로드시 데이타필드명
			uploadCore.uploadDataFieldName = params.uploadDataFieldName as String;
			
			//Controls the HTTP form submission method : post or get
			uploadCore.method = params.method as String;
			
			//An object containing data to be transmitted with the URL request
			//꼭, "{'id':'jidolstar','pw':'1111'}" 형태여야 정상적으로 동작한다.
			if( params.variables )
			{
				try
				{
					uploadCore.variables = objectToURLVariables( stringToObject( params.variables ) );
				}
				catch( e:Error )
				{
					throw new Error( "[Error] requestHeaders setup fail. message="+e.message );
				}
			}
						
			//The array of HTTP request headers to be appended to the HTTP request
			//꼭  "{'Content-Type':'text/html', .... }" 형태여야 정상적으로 동작한다.
			if( params.requestHeaders )
			{
				try
				{				
					var obj:Object = stringToObject( params.requestHeaders );
					var requestHeaders:Array = objectToRequestHeaders( obj );
					uploadCore.requestHeaders = requestHeaders;
				}
				catch( e:Error )
				{
					throw new Error( "[Error] requestHeaders setup fail. message="+e.message );
				}
			}

			
			//The MIME content type of the content in the the data property.
			if( params.contentType )
			{
				uploadCore.contentType = params.contentType as String;
			}

			//파일 업로드 경로
			uploadCore.uploadURL = params.uploadURL as String;
			
			
			//FAService Javascipt 인스턴스 명 
			faservice.faServiceJSInstanceName = params.faServiceJSInstanceName as String;

			//Flex-Javascript 통신 시작 
			faservice.run();	
		}
		
		/**
		 * 파일 브라우징 
		 */ 
		public function browse():void
		{
			uploadCore.browse();
		}
		
		/**
		 * 파일 업로드 중지 
		 */ 
		public function stop():void
		{
			uploadCore.stop();
		}
		
		/**
		 * {'id':'jidolstar','pw':'1111'} 형태를
		 * {"id":"jidolstar","pw":"1111"} 로 바꾸고 
		 * 이것은 JSON.decode를 이용해 Object로 변환한다. 
		 */ 
		private function stringToObject( str:String ):Object
		{
			var req:RegExp = new RegExp("'","g");			
			var str:String = str.replace( req, "\"");
			var obj:Object = JSON.decode( str );
			return obj;			
		}
		
		/**
		 * Object를 URLVariables 로 만들어 준다.
		 */ 
		private function objectToURLVariables( obj:Object ):URLVariables
		{
			if( obj == null ) return null;
			var variables:URLVariables = new URLVariables;
			for( var key:String in obj )
			{
				variables[key] = obj[key];
			}
			return variables;
		}
		
		/**
		 * Object를 가지고  URLRequestHeader를 만들어 리스트를 반환한다.
		 */  
		private function objectToRequestHeaders( obj:Object ):Array
		{
			if( obj == null ) return null;
			var arr:Array = new Array;
			for( var key:String in obj )
			{
				arr.push( new URLRequestHeader( key, obj[key] ) );
			}
			return (arr.length==0)?null:arr;
		}

	}
}