package com.jidolstar.fileupload
{
	import com.jidolstar.fa.*;
	
	import flash.events.EventDispatcher;

	/**
	 * Flex에서 발생한 이벤트가 있을 때, 등록된 Javascript Event Listener를 호출할 함수들로 구성된 클래스이다.
	 * 
	 * @author 지돌스타
	 * @since 2008.10.26
	 */ 
	internal class Flex2JSFunctions
	{
		public var _target:EventDispatcher;
		public var callerList:Array;
		
		public function Flex2JSFunctions():void
		{
		}
		
		public function get target():EventDispatcher
		{
			return _target;
		}
		
		public function set target( value:EventDispatcher ):void
		{
			if( value == _target ) return;
			_target = value;
			
			if( _target != null )
			{
				//이 부분을 생성자에 넣으면 안된다. 왜냐하면 map이 생성되지 않은 상태에서 Flex2JSCaller가 생성되기 때문이다.
				callerList = [
								new Flex2JSCaller( _target, FileUploadEvent.BAN_FILE_EXTENSION, fileExtensionError ),
								new Flex2JSCaller( _target, FileUploadEvent.CANCEL, cancel ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_START, uploadStart ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_STEP, uploadStep ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_END, uploadEnd ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_FAIL, uploadFail ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_START_ALL, uploadStartAll ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_STOP_ALL, uploadStopAll ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_END_ALL, uploadEndAll ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_FILE_COUNT_ERROR, uploadFileCountError ),
								new Flex2JSCaller( _target, FileUploadEvent.UPLOAD_FILE_SIZE_ERROR, uploadFileSizeError ),
				  			];
			}
		}
		
		/**
		 * fileExtensionError()
		 * eventName fileuploader_banFileExtension
		 * return {filename:파일이름,banFileExtension:제한한 파일 확장자, banFileExtensions:등록되어 있는 제한 파일 확장자}
		 */ 
		private function fileExtensionError( event:FileUploadEvent ):Object
		{
			return event.data;
		}

		/**
		 * cancel()
		 * eventName fileuploader_cancel
		 * return null
		 */ 
		private function cancel( event:FileUploadEvent ):Object
		{
			return null;
		}

		/**
		 * uploadStart()
		 * eventName fileuploader_start
		 * return {"filename":파일이름, "bytesTotal":파일사이즈};
		 */ 		
		private function uploadStart( event:FileUploadEvent ):Object
		{
			var fileDetail:FileDetail = event.data.currentFileDetail;
			var data:Object = {	"filename":fileDetail.filename, 
								"bytesTotal":fileDetail.bytesTotal};
			return data;
		}

		/**
		 * uploadStep()
		 * eventName fileuploader_step
		 * return {"filename":파일명, "bytesTotal":파일사이즈, "bytesLoaded":올라간용량};
		 */ 		
		private function uploadStep( event:FileUploadEvent ):Object
		{
			var fileDetail:FileDetail = event.data.currentFileDetail;
			var data:Object = {"filename":fileDetail.filename, 
							   "bytesTotal":fileDetail.bytesTotal, 
							   "bytesLoaded":fileDetail.bytesLoaded};
			return data;
		}

		/**
		 * uploadEnd()
		 * eventName fileuploader_end
		 * return {"filename":파일명, "bytesTotal":파일사이즈, "uploadCompleteData":{"uploadedFilePath": "http://jidolstar.com/upload/uploaded_filename.ext", "uploadedFilename":"uploaded_filename.ext"}}
		 */ 		
		private function uploadEnd( event:FileUploadEvent ):Object
		{
			var fileDetail:FileDetail = event.data.currentFileDetail;
			var data:Object = { "filename":fileDetail.filename, 
								"bytesTotal":fileDetail.bytesTotal, 
								"uploadCompleteData":fileDetail.uploadCompleteData};
			return data;
			
		}

		/**
		 * uploadFail()
		 * eventName fileuploader_fail
		 * return {"filename":파일명, "bytesTotal":파일사이즈, "msg":에러메시지};
		 */ 		
		private function uploadFail( event:FileUploadEvent ):Object
		{
			var fileDetail:FileDetail = event.data.currentFileDetail;
			var data:Object = {	"filename":fileDetail.filename, 
								"bytesTotal":fileDetail.bytesTotal, 
								"msg":fileDetail.message};
			return data;
		}
		
		/**
		 * uploadStartAll()
		 * eventName fileuploader_startAll
		 * return {"totalCount":모든 파일 수, "totalSize":전체 파일 크기 };
		 */ 		
		private function uploadStartAll( event:FileUploadEvent ):Object
		{
			var fileDetailList:FileDetailList = event.data as FileDetailList;
			var data:Object = {	"totalCount":fileDetailList.fileTotalCount, 
								"totalSize":fileDetailList.fileTotalSize};
			return data;
		}	
		
		/**
		 * uploadStopAll()
		 * eventName fileuploader_stopAll
		 * return {"totalCount":모든 파일 수, "totalSize":전체 파일 크기 };
		 */ 		
		private function uploadStopAll( event:FileUploadEvent ):Object
		{
			var fileDetailList:FileDetailList = event.data as FileDetailList;
			var data:Object = {	"totalCount":fileDetailList.fileTotalCount, 
								"totalSize":fileDetailList.fileTotalSize};
			return data;
		}
	
		/**
		 * uploadEndAll()
		 * eventName fileuploader_endAll
		 * return {"failCount":실패수, "endCount":성공수, "totalCount":모든 수 };
		 */ 		
		private function uploadEndAll( event:FileUploadEvent ):Object
		{
			var fileDetailList:FileDetailList = event.data as FileDetailList;
			var data:Object = {	"failCount":fileDetailList.uploadFailCount, 
								"endCount":fileDetailList.uploadEndCount, 
								"totalCount":fileDetailList.fileTotalCount, 
								"totalSize":fileDetailList.fileTotalSize};
			return data;
		}
		
		/**
		 * uploadFileCountError()
		 * eventName fileuploader_fileSizeError
		 * return {"filename":파일이름, "bytesTotal":파일크기, "maxOneFileSize": 지정된 파일 크기};
		 */ 		
		private function uploadFileSizeError( event:FileUploadEvent ):Object
		{
			var fileDetail:FileDetail = event.data.currentFileDetail;
			var data:Object = {	"filename":fileDetail.filename, 
								"bytesTotal":fileDetail.bytesTotal, 
								"maxOneFileSize": event.data.maxOneFileSize};
			return data;
		}

		/**
		 * uploadFileCountError()
		 * eventName fileuploader_fileCountError
		 * return {"totalCount":전체파일수, "maxFileCount": 최대파일수 };
		 */ 		
		private function uploadFileCountError( event:FileUploadEvent ):Object
		{
			var fileDetailList:FileDetailList = event.data as FileDetailList;
			var data:Object = {	"totalCount":fileDetailList.fileTotalCount, 
								"maxFileCount": fileDetailList.maxFileCount };
			return data;
		}

	}
}