/**
 * JS-Flex간 통신하기 위해 사용한다.
 * 
 * @author jidolstar (jidolstar@gmail.com)
 * @since 2008.10.09.
 * @version 2008.10.13.( Last Modified )
 * 
 * @param swfObjName 별지도 SWF 객체 이름
 * 
 * @example
 *  아래는 swfobject 2.1와 FAService를 활용하는 예제이다.
 * 
 *  var mapSWFObjectName = "mapSWFObj"; 							//*.swf파일 객체의 id와 name이다. FAService에 전달하여 생성되는 swf에 접근하는데 쓰인다.
 * 	var mapFAService = new FAService( mapSWFObjectName );
 * 	mapFAService.addEventListener( "ready", onReady, 0  );				//이벤트를 추가할 수 있다. ready이벤트 후에 call()함수를 호출할 수 있다.
 * 	mapFAService.addEventListener( "mapevent_moveend", onMove, 0 ); 	//이벤트를 등록한다. 이벤트가 발생하면 onMove가 호출된다.
 *  var flashvars = {faServiceInstanceName:"mapFAService"}; 			//(필수) faServiceInstanceName으로 FAService의 인스턴스 명을 전달한다. 이 작업을 해야 Flex에서 FAService의 인스턴스에 접근할 수 있다.
 *  var params = {allowscriptaccess : "samedomain"};					//(필수) allowsriptaccess를 always나 samedomain으로 설정해야한다. never로 하면 Flex-JS간 통신이 단절된다.
 *  var attributes = { id: mapSWFObjectName, name: mapSWFObjectName }; 	//(필수) id, name을 지정해야한다. 여기서 지정된 이름은 반드시 FAService()생성시 인자로 넘겨주어야 Flex-JS간에 통신이 가능해진다.  
 *  swfobject.embedSWF( "starplmap.swf", "divMap", "100%", "100%", "9", "expressInstall.swf", flashvars, params, attributes);
 * 	function onReady()
 * 	{
 * 		var objJSON = { ra: 12, dec: 0 };
 * 		objJSON = mapFAService.call("moveTo", objJSON ); 			//(필수)mapFAService의 call함수는 반드시 ready 이벤트 발생후 호출한다. mapFAService의 isReady로도 확인이 가능하다. 
 * 	}
 * 	function onMove( objJSON )										//(필수)이벤트 리스너 함수는 ready 이벤트를 제외하고 반드시 1개 Object형을 받는다. 없으면 null값이 온다.
 * 	{
 * 		alert( "onMove :" + objJSON.ra );
 * 		mapFAService.removeEventListener( "mapevent_moveend", onMove ); //필요없는 이벤트 리스너를 언제든지 삭제할 수 있다.
 * 	} 
 */
function FAService( swfObjName  )
{
	if( swfObjName == null ) return;
	this.swfObjectName = swfObjName;
	this.readyList = new Array();
	this.prevReadyAddedEventListenerList = new Array();
	this.isShowErrorMsg = true;
	this.isReady = false;
}

/**
 * SWF 무비 객체에 대한 접근자를 얻어오는데 사용한다.
 * 사용자가 직접 사용하지 않는다.
 */ 
FAService.prototype.getMovie = function() 
{
	return document.getElementById(this.swfObjectName);
	//return (navigator.appName.indexOf("Microsoft") != -1)?window[swfObject]:document[swfObject];
}	

////////////////////////////////////////////////////////////////
//	Flex to Javascript 
////////////////////////////////////////////////////////////////

/**
 * Flex에서 이 함수를 이용하여 ExternalInterface를 이용한 통신이 가능한지 확인한다.
 * 사용자가 직접 쓰는 경우는 없다.
 */
FAService.prototype.checkExternalInterface = function()
{
	var movie = this.getMovie();
	if(movie == undefined) return false;
	return movie.checkExternalInterface();
}

/**
 * Flex-JS간 통신이 가능해질때 이 함수가 호출된다.
 * 사용자가 직접 사용하지 않는다.
 */
FAService.prototype.ready = function()
{
	//한번 호출디면 더이상 호출되지 않는다.
	if( this.isReady == true ) return;
	
	//미루어진 이벤트 등록 
	var movie = this.getMovie();
	for( i=0; i < this.prevReadyAddedEventListenerList.length; i++ )
	{
		listener = this.prevReadyAddedEventListenerList[i];
		movie.addEventListener( listener.eventName, listener.listenerName, listener.priority );
	}
	this.prevReadyAddedEventListenerList = null;
	
	//준비완료 등록된 함수 호출  
	for( i=0; i < this.readyList.length; i++ )
	{
		func = this.readyList[i];
		eval('('+func+')')();
	}
	this.readyList = null;
	this.isReady = true;
}

/**
 * 잘못된 이벤트명으로 이벤트 리스너를 등록할 경우 발생한다.
 * 사용자가 직접 호출하지 않는다.
 * @param errorMessage 에러메세지
 */
FAService.prototype.wrongEventListener = function( errorMessage )
{
	if( this.isShowErrorMsg  )
		alert(errorMessage);
}

/**
 * 잘못된 호출명 또는 잘못된 json 인자를 사용했을 경우 발생한다.
 * 사용자가 직접 호출하지 않는다.
 * @param errorMessage 에러메세지
 */
FAService.prototype.wrongCall = function( errorMessage )
{
	if( this.isShowErrorMsg  )
		alert(errorMessage);
}

/**
 * addEventListener로 추가된 이벤트 리스너를 호출한다.
 * 사용자가 직접 호출하지 않는다.
 * @param listenerName 호출할 이벤트 리스너의 함수명
 * @param objJSON JSON 형태의 반환값
 */
FAService.prototype.calladdedEventListener = function( listenerName, objJSON )
{
	listener = eval(listenerName);
	listener(objJSON);
}


////////////////////////////////////////////////////////////////
//	Javascript to Flex  
////////////////////////////////////////////////////////////////

/**
 * Flex내 특정 기능을 수행할 때 사용한다.
 * @param callName Flex내에서 허락한 호출명이다. 허락되지 않은 호출은 무시된다.
 * @param objJSON 입력값, JSON Object 형태. 호출명에 따라서 입력해야하는 값이 다르다.(사용자 Flex내부 정의 나름)
 * @return JSON Object 형태로 반환해준다, 호출명에 따라서 반환값이 다르다.(사용자 Flex 내부 정의 나름)
 * ex) 
 * 		var objJSON = { ra: 12, dec: 0 };
 *		FAService.call( "moveTo", objJSON );
 */
FAService.prototype.call = function( callName, objJSON )
{
	var movie = this.getMovie();
	if(movie == undefined)
	{
		alert( "[Error]After a 'ready' event, you can call call() method.")
		return;
	}
	return movie.call( callName, objJSON );
}

/**
 * 이벤트 리스너를 등록할 때 사용한다.
 * @param eventName 이벤트 명이다. 잘못 등록하면 wrongEventListener() 메소드가 호출된다.
 * @param listener 해당 이벤트가 발생할때 호출할 함수이다.
 * @param priority 우선순위이다. 값이 클수록 우선순위가 높다. 만약 같은 이벤트명에 대한 listener가 2개 등록된다면 우선순위가 높은 것이 먼저 호출된다. 같은 우선순위면 등록순으로 호출된다.
 * 
 * ex)
 * 		var faService = new FAService(swfObjectName);
 * 		faService.addEventListener( "mapevent_movestart", onMoveStart, 0 );
 */
FAService.prototype.addEventListener = function( eventName, listener, priority )
{
	//Flex-JS 통신이 준비가 되어 있지 않은 경우 예약처리 해준다.
	if( this.isReady == false )
	{
		//ready 이벤트에 대한 처리
		//ready() 함수가 자동으로 호출될때 listener를 호출해준다. 
		if( eventName == "ready" )
		{
			this.readyList.push(listener);
			return;
		}
		//다른 이벤트에 대한 처리
		//ready() 함수가 자동으로 호출될때 listener를 호출해준다.
		else
		{
			this.prevReadyAddedEventListenerList.push( {"eventName":eventName, "listenerName":listener, "priority":priority} );
			return;
		} 
	}
	//Flex-JS 통신이 되고 있는 상태이므로 무시된다.
	if( eventName == "ready" )
	{
		return;
	}
	var movie = this.getMovie();
	if(movie == undefined) return; 
	movie.addEventListener( eventName, listener, priority );
}

/**
 * 이벤트 리스너를 삭제할 때 사용한다.
 * @param eventName 이벤트 명이다. 서비스 하고 있는 이벤트명을 제대로 써야 동작한다.
 * @param listener 삭제할 이벤트 리스너 함수이다.
 * 
 * ex)
 * 		var faService = new FAService(swfObjectName);
 * 		faService.removeEventListener( "movestart", onMoveStart );
 * 		function onMoveStart( event )
 */
FAService.prototype.removeEventListener = function( eventName, listener )
{
	var movie = this.getMovie();
	if(movie == undefined) return;
	movie.removeEventListener( eventName, listener);
}




